# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import re
from unittest import mock

import ddt
import requests

import manilaclient
from manilaclient.common import httpclient
from manilaclient import exceptions
from manilaclient.tests.unit import utils

fake_user_agent = "fake"

fake_response = utils.TestResponse({
    "status_code": 200,
    "text": '{"hi": "there"}',
})
mock_request = mock.Mock(return_value=(fake_response))

bad_400_response = utils.TestResponse({
    "status_code": 400,
    "text": '{"error": {"message": "n/a", "details": "Terrible!"}}',
})
bad_400_request = mock.Mock(return_value=(bad_400_response))

bad_401_response = utils.TestResponse({
    "status_code": 401,
    "text": '{"error": {"message": "FAILED!", "details": "DETAILS!"}}',
})
bad_401_request = mock.Mock(return_value=(bad_401_response))

bad_500_response = utils.TestResponse({
    "status_code": 500,
    "text": '{"error": {"message": "FAILED!", "details": "DETAILS!"}}',
})
bad_500_request = mock.Mock(return_value=(bad_500_response))

retry_after_response = utils.TestResponse({
    "status_code": 413,
    "text": '',
    "headers": {
        "retry-after": "5"
    },
})
retry_after_mock_request = mock.Mock(return_value=retry_after_response)

retry_after_no_headers_response = utils.TestResponse({
    "status_code": 413,
    "text": '',
})
retry_after_no_headers_mock_request = mock.Mock(
    return_value=retry_after_no_headers_response)

retry_after_non_supporting_response = utils.TestResponse({
    "status_code": 403,
    "text": '',
    "headers": {
        "retry-after": "5"
    },
})
retry_after_non_supporting_mock_request = mock.Mock(
    return_value=retry_after_non_supporting_response)


def get_authed_client(endpoint_url="http://example.com", retries=0):
    cl = httpclient.HTTPClient(endpoint_url, "token", fake_user_agent,
                               retries=retries, http_log_debug=True,
                               api_version=manilaclient.API_MAX_VERSION)
    return cl


@ddt.ddt
class ClientTest(utils.TestCase):

    def setUp(self):
        super(ClientTest, self).setUp()
        self.max_version = manilaclient.API_MAX_VERSION
        self.max_version_str = self.max_version.get_string()

    @ddt.data(
        "http://manila.example.com/v2/b2d18606-2673-4965-885a-4f5a8b955b9b",
        "http://manila.example.com/v1",
        "http://manila.example.com/share/v2.22/",
        "http://manila.example.com/share/v1/"
        "b2d18606-2673-4965-885a-4f5a8b955b9b",
        "http://10.10.10.10:3366/v1",
        "http://10.10.10.10:3366/v2/b2d18606-2673-4965-885a-4f5a8b955b9b",
        "http://manila.example.com:3366/v1.1/",
        "http://manila.example.com:3366/v2/"
        "b2d18606-2673-4965-885a-4f5a8b955b9b")
    def test_get(self, endpoint_url):
        cl = get_authed_client(endpoint_url)

        @mock.patch.object(requests, "request", mock_request)
        @mock.patch('time.time', mock.Mock(return_value=1234))
        def test_get_call():
            resp, body = cl.get("/hi")
            headers = {
                "X-Auth-Token": "token",
                "User-Agent": fake_user_agent,
                cl.API_VERSION_HEADER: self.max_version_str,
                'Accept': 'application/json',
            }
            mock_request.assert_called_with(
                "GET",
                endpoint_url + "/hi",
                headers=headers,
                **self.TEST_REQUEST_BASE)
            # Automatic JSON parsing
            self.assertEqual(body, {"hi": "there"})
            self.assertEqual(re.split(r'/v[0-9]+[\.0-9]*',
                                      endpoint_url)[0] + "/", cl.base_url)

        test_get_call()

    def test_get_retry_500(self):
        cl = get_authed_client(retries=1)

        self.requests = [bad_500_request, mock_request]

        def request(*args, **kwargs):
            next_request = self.requests.pop(0)
            return next_request(*args, **kwargs)

        @mock.patch.object(requests, "request", request)
        @mock.patch('time.time', mock.Mock(return_value=1234))
        def test_get_call():
            resp, body = cl.get("/hi")

        test_get_call()
        self.assertEqual(self.requests, [])

    def test_retry_limit(self):
        cl = get_authed_client(retries=1)

        self.requests = [bad_500_request, bad_500_request, mock_request]

        def request(*args, **kwargs):
            next_request = self.requests.pop(0)
            return next_request(*args, **kwargs)

        @mock.patch.object(requests, "request", request)
        @mock.patch('time.time', mock.Mock(return_value=1234))
        def test_get_call():
            resp, body = cl.get("/hi")

        self.assertRaises(exceptions.ClientException, test_get_call)
        self.assertEqual(self.requests, [mock_request])

    def test_get_no_retry_400(self):
        cl = get_authed_client(retries=0)

        self.requests = [bad_400_request, mock_request]

        def request(*args, **kwargs):
            next_request = self.requests.pop(0)
            return next_request(*args, **kwargs)

        @mock.patch.object(requests, "request", request)
        @mock.patch('time.time', mock.Mock(return_value=1234))
        def test_get_call():
            resp, body = cl.get("/hi")

        self.assertRaises(exceptions.BadRequest, test_get_call)
        self.assertEqual(self.requests, [mock_request])

    def test_get_retry_400_socket(self):
        cl = get_authed_client(retries=1)

        self.requests = [bad_400_request, mock_request]

        def request(*args, **kwargs):
            next_request = self.requests.pop(0)
            return next_request(*args, **kwargs)

        @mock.patch.object(requests, "request", request)
        @mock.patch('time.time', mock.Mock(return_value=1234))
        def test_get_call():
            resp, body = cl.get("/hi")

        test_get_call()
        self.assertEqual(self.requests, [])

    def test_get_with_retries_none(self):
        cl = get_authed_client(retries=None)

        @mock.patch.object(requests, "request", bad_401_request)
        def test_get_call():
            resp, body = cl.get("/hi")

        self.assertRaises(exceptions.Unauthorized, test_get_call)

    @ddt.data(
        "http://manila.example.com/v1/b2d18606-2673-4965-885a-4f5a8b955b9b",
        "http://manila.example.com/v1",
        "http://manila.example.com/share/v2.1/",
        "http://manila.example.com/share/v1/"
        "b2d18606-2673-4965-885a-4f5a8b955b9b",
        "http://10.10.10.10:3366/v1.1",
        "http://10.10.10.10:3366/v2/b2d18606-2673-4965-885a-4f5a8b955b9b",
        "http://manila.example.com:3366/v2.22/",
        "http://manila.example.com:3366/v1/"
        "b2d18606-2673-4965-885a-4f5a8b955b9b")
    def test_post(self, endpoint_url):
        cl = get_authed_client(endpoint_url)

        @mock.patch.object(requests, "request", mock_request)
        def test_post_call():
            cl.post("/hi", body=[1, 2, 3])
            headers = {
                "X-Auth-Token": "token",
                "Content-Type": "application/json",
                'Accept': 'application/json',
                "X-Openstack-Manila-Api-Version": self.max_version_str,
                "User-Agent": fake_user_agent
            }
            mock_request.assert_called_with(
                "POST",
                endpoint_url + "/hi",
                headers=headers,
                data='[1, 2, 3]',
                **self.TEST_REQUEST_BASE)
            self.assertEqual(re.split(r'/v[0-9]+[\.0-9]*',
                                      endpoint_url)[0] + "/", cl.base_url)

        test_post_call()



# Copyright (c) 2015 Clinton Knight.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes
from manilaclient.v2 import scheduler_stats


class PoolTest(utils.TestCase):

    def setUp(self):
        super(PoolTest, self).setUp()

        self.name = 'fake_host@fake_backend#fake_pool'
        info = {
            'name': self.name,
            'host': 'fake_host',
            'backend': 'fake_backend',
            'pool': 'fake_pool',
        }
        self.resource_class = scheduler_stats.Pool(manager=self, info=info)

    def test_get_repr_of_share_server(self):
        self.assertEqual('<Pool: %s>' % self.name, repr(self.resource_class))


class PoolManagerTest(utils.TestCase):

    def setUp(self):
        super(PoolManagerTest, self).setUp()
        self.manager = scheduler_stats.PoolManager(fakes.FakeClient())

    @mock.patch.object(scheduler_stats.PoolManager, '_list', mock.Mock())
    def test_list(self):
        self.manager.list(detailed=False)
        self.manager._list.assert_called_once_with(
            scheduler_stats.RESOURCES_PATH,
            scheduler_stats.RESOURCES_NAME)

    @mock.patch.object(scheduler_stats.PoolManager, '_list', mock.Mock())
    def test_list_detail(self):
        self.manager.list()
        self.manager._list.assert_called_once_with(
            scheduler_stats.RESOURCES_PATH + '/detail',
            scheduler_stats.RESOURCES_NAME)

    @mock.patch.object(scheduler_stats.PoolManager, '_list', mock.Mock())
    def test_list_with_one_search_opt(self):
        host = 'fake_host'
        query_string = "?host=%s" % host

        self.manager.list(detailed=False, search_opts={'host': host})

        self.manager._list.assert_called_once_with(
            scheduler_stats.RESOURCES_PATH + query_string,
            scheduler_stats.RESOURCES_NAME)

    @mock.patch.object(scheduler_stats.PoolManager, '_list', mock.Mock())
    def test_list_detail_with_two_search_opts(self):
        host = 'fake_host'
        backend = 'fake_backend'
        query_string = "?backend=%s&host=%s" % (backend, host)

        self.manager.list(search_opts={'host': host, 'backend': backend})

        self.manager._list.assert_called_once_with(
            scheduler_stats.RESOURCES_PATH + '/detail' + query_string,
            scheduler_stats.RESOURCES_NAME)



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt

from manilaclient.tests.functional import base


@ddt.ddt
class SharesMetadataReadWriteTest(base.BaseTestCase):

    def setUp(self):
        super(SharesMetadataReadWriteTest, self).setUp()
        self.share = self.create_share(client=self.get_user_client())

    def test_set_metadata_in_share_creation(self):
        md = {"key1": "value1", "key2": "value2"}

        # Create share with metadata
        share = self.create_share(metadata=md, client=self.get_user_client())

        # Read share metadata
        metadata = self.user_client.get_share_metadata(share["id"])

        # Verify share metadata
        self.assertEqual(2, len(metadata))
        self.assertIn('key1', metadata)
        self.assertIn('key2', metadata)
        self.assertEqual(md['key1'], metadata['key1'])
        self.assertEqual(md['key2'], metadata['key2'])

    def test_set_and_get_metadata(self):
        # Create share
        share = self.create_share(
            cleanup_in_class=False, client=self.get_user_client())

        # Set share metadata
        md = {"key3": "value3", "key4": "value4"}
        self.user_client.set_share_metadata(share["id"], md)

        # Read share metadata
        metadata = self.user_client.get_share_metadata(share["id"])

        # Verify share metadata
        self.assertEqual(2, len(metadata))
        self.assertIn('key3', metadata)
        self.assertIn('key4', metadata)
        self.assertEqual(md['key3'], metadata['key3'])
        self.assertEqual(md['key4'], metadata['key4'])

    def test_set_and_delete_metadata(self):
        # Create share
        share = self.create_share(
            cleanup_in_class=False, client=self.get_user_client())

        # Set share metadata
        md = {"key3": "value3", "key4": "value4"}
        self.user_client.set_share_metadata(share["id"], md)

        # Unset share metadata
        self.user_client.unset_share_metadata(share["id"], list(md.keys()))

        # Verify deletion of share metadata
        metadata = self.user_client.get_share_metadata(share["id"])
        self.assertEqual({}, metadata)

    def test_set_and_add_metadata(self):
        md = {'key5': 'value5'}

        # Create share with metadata
        share = self.create_share(
            metadata=md, cleanup_in_class=False, client=self.get_user_client())

        # Set share metadata
        self.user_client.set_share_metadata(share["id"], {'key6': 'value6'})
        self.user_client.set_share_metadata(share["id"], {'key7': 'value7'})

        # Read share metadata
        metadata = self.user_client.get_share_metadata(share["id"])

        # Verify share metadata
        self.assertEqual(3, len(metadata))
        for i in (5, 6, 7):
            key = 'key%s' % i
            self.assertIn(key, metadata)
            self.assertEqual('value%s' % i, metadata[key])

    def test_set_and_replace_metadata(self):
        md = {'key8': 'value8'}

        # Create share with metadata
        share = self.create_share(
            metadata=md, cleanup_in_class=False, client=self.get_user_client())

        # Set share metadata
        self.user_client.set_share_metadata(share["id"], {'key9': 'value9'})

        # Replace all existing share metadata
        self.user_client.update_all_share_metadata(
            share["id"], {'key10': 'value10'})

        # Read share metadata
        metadata = self.user_client.get_share_metadata(share["id"])

        # Verify share metadata
        self.assertEqual(1, len(metadata))
        self.assertIn('key10', metadata)
        self.assertEqual('value10', metadata['key10'])

    @ddt.data(
        {"k": "value"},
        {"k" * 255: "value"},
        {"key": "v"},
        {"key": "v" * 1023})
    def test_set_metadata_min_max_sizes_of_keys_and_values(self, metadata):
        # Set share metadata
        self.user_client.set_share_metadata(self.share["id"], metadata)

        # Read share metadata
        get = self.user_client.get_share_metadata(self.share["id"])

        # Verify share metadata
        key = list(metadata.keys())[0]
        self.assertIn(key, get)
        self.assertEqual(metadata[key], get[key])

    @ddt.data(
        {"k": "value"},
        {"k" * 255: "value"},
        {"key": "v"},
        {"key": "v" * 1023})
    def test_update_metadata_min_max_sizes_of_keys_and_values(self, metadata):
        # Update share metadata
        self.user_client.update_all_share_metadata(self.share["id"], metadata)

        # Read share metadata
        get = self.user_client.get_share_metadata(self.share["id"])

        # Verify share metadata
        self.assertEqual(len(metadata), len(get))
        for key in metadata:
            self.assertIn(key, get)
            self.assertEqual(metadata[key], get[key])



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient import api_versions
from manilaclient import base
from manilaclient.common.apiclient import base as common_base

RESOURCE_PATH_LEGACY = '/os-services'
RESOURCE_PATH = '/services'
RESOURCE_NAME = 'services'


class Service(common_base.Resource):

    def __repr__(self):
        return "<Service: %s>" % self.id

    def server_api_version(self, **kwargs):
        """Get api version."""
        return self.manager.api_version(self, kwargs)


class ServiceManager(base.Manager):
    """Manage :class:`Service` resources."""
    resource_class = Service

    def _do_list(self, search_opts=None, resource_path=RESOURCE_PATH):
        """Get a list of all services.

        :rtype: list of :class:`Service`
        """
        query_string = self._build_query_string(search_opts)
        return self._list(resource_path + query_string, RESOURCE_NAME)

    @api_versions.wraps("1.0", "2.6")
    def list(self, search_opts=None):
        return self._do_list(
            search_opts=search_opts, resource_path=RESOURCE_PATH_LEGACY)

    @api_versions.wraps("2.7")  # noqa
    def list(self, search_opts=None):  # noqa
        return self._do_list(
            search_opts=search_opts, resource_path=RESOURCE_PATH)

    def _do_enable(self, host, binary, resource_path=RESOURCE_PATH):
        """Enable the service specified by hostname and binary."""
        body = {"host": host, "binary": binary}
        return self._update("%s/enable" % resource_path, body)

    @api_versions.wraps("1.0", "2.6")
    def enable(self, host, binary):
        return self._do_enable(host, binary, RESOURCE_PATH_LEGACY)

    @api_versions.wraps("2.7")  # noqa
    def enable(self, host, binary):  # noqa
        return self._do_enable(host, binary, RESOURCE_PATH)

    def _do_disable(self, host, binary, resource_path=RESOURCE_PATH):
        """Disable the service specified by hostname and binary."""
        body = {"host": host, "binary": binary}
        return self._update("%s/disable" % resource_path, body)

    @api_versions.wraps("1.0", "2.6")
    def disable(self, host, binary):
        return self._do_disable(host, binary, RESOURCE_PATH_LEGACY)

    @api_versions.wraps("2.7")  # noqa
    def disable(self, host, binary):  # noqa
        return self._do_disable(host, binary, RESOURCE_PATH)

    def server_api_version(self, url_append=""):
        """Returns the API Version supported by the server.

        :param url_append: String to append to url to obtain specific version
        :return: Returns response obj for a server that supports microversions.
                 Returns an empty list for Kilo and prior Manila servers.
        """
        try:
            return self._get_with_base_url(url_append, 'versions')
        except LookupError:
            return []



# Copyright 2016 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient.tests.unit import utils


class ShareServersV1Test(utils.TestCase):

    def test_import_v1_share_servers_module(self):
        try:
            from manilaclient.v1 import share_servers
        except Exception as e:
            msg = ("module 'manilaclient.v1.share_servers' cannot be imported "
                   "with error: %s") % str(e)
            assert False, msg
        for cls in ('ShareServer', 'ShareServerManager'):
            msg = "Module 'share_servers' has no '%s' attr." % cls
            self.assertTrue(hasattr(share_servers, cls), msg)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import time

from tempest.lib.cli import base
from tempest.lib.common.utils import data_utils

from manilaclient import config

CONF = config.CONF


class OSCClientTestBase(base.ClientTestBase):
    """Base class for OSC manila functional tests"""

    def _get_clients(self):

        return base.CLIClient(
            username=CONF.admin_username or CONF.username,
            password=CONF.admin_password or CONF.password,
            tenant_name=CONF.admin_tenant_name or CONF.tenant_name,
            uri=CONF.admin_auth_url or CONF.auth_url,
            cli_dir=CONF.manila_exec_dir,
            insecure=CONF.insecure,
            project_domain_name=(CONF.admin_project_domain_name or
                                 CONF.project_domain_name or None),
            project_domain_id=(CONF.admin_project_domain_id or
                               CONF.project_domain_id or None),
            user_domain_name=(CONF.admin_user_domain_name or
                              CONF.user_domain_name or None),
            user_domain_id=(CONF.admin_user_domain_id or
                            CONF.user_domain_id or None)
        )

    def _get_property_from_output(self, output):
        """Creates a dictionary from the given output"""
        obj = {}
        items = self.parser.listing(output)
        for item in items:
            obj[item['Field']] = str(item['Value'])
        return obj

    def _wait_for_object_status(self, object_name, object_id, status,
                                timeout=CONF.build_timeout,
                                interval=CONF.build_interval):
        """Waits for a object to reach a given status."""

        start_time = time.time()
        while time.time() - start_time < timeout:
            if status == self.openstack(
                    '%(obj)s show -c status -f value %(id)s'
                    % {'obj': object_name,
                       'id': object_id}).rstrip():
                break
            time.sleep(interval)
        else:
            self.fail("%s %s did not reach status %s after %d seconds."
                      % (object_name, object_id, status, timeout))

    def check_object_deleted(self, object_name, object_id,
                             timeout=CONF.build_timeout,
                             interval=CONF.build_interval):
        """Check that object deleted successfully"""

        cmd = '%s list -c ID -f value' % object_name
        start_time = time.time()
        while time.time() - start_time < timeout:
            if object_id not in self.openstack(cmd):
                break
            time.sleep(interval)
        else:
            self.fail("%s %s not deleted after %d seconds."
                      % (object_name, object_id, timeout))

    def openstack(self, action, flags='', params='', fail_ok=False,
                  merge_stderr=False):
        """Executes openstack command for given action"""

        if '--os-share-api-version' not in flags:
            flags = (
                flags + '--os-share-api-version %s'
                % CONF.max_api_microversion)

        return self.clients.openstack(action, flags=flags, params=params,
                                      fail_ok=fail_ok,
                                      merge_stderr=merge_stderr)

    def listing_result(self, object_name, command):
        """Returns output for the given command as list of dictionaries"""

        output = self.openstack(object_name, params=command)
        result = self.parser.listing(output)
        return result

    def dict_result(self, object_name, command):
        """Returns output for the given command as dictionary"""

        output = self.openstack(object_name, params=command)
        result_dict = self._get_property_from_output(output)
        return result_dict

    def create_share(self, share_protocol=None, size=None, name=None,
                     snapshot_id=None, properties=None, share_network=None,
                     description=None, public=False, share_type=None,
                     availability_zone=None, share_group=None,
                     add_cleanup=True):

        name = name or data_utils.rand_name('autotest_share_name')
        # share_type = dhss_false until we have implemented
        # share network commands for osc
        share_type = share_type or 'dhss_false'

        cmd = ('create '
               '%(protocol)s %(size)s %(name)s %(desc)s %(public)s %(stype)s'
               % {'protocol': share_protocol or 'NFS',
                  'size': size or '1',
                  'name': '--name %s' % name,
                  'desc': '--description %s' % description,
                  'public': '--public %s' % public,
                  'stype': '--share-type %s' % share_type})

        if snapshot_id:
            cmd = cmd + ' --snapshot-id %s' % snapshot_id
        if properties:
            for key, value in properties.items():
                cmd = (cmd + ' --property %(key)s=%(value)s'
                       % {'key': key, 'value': value})
        if share_network:
            cmd = cmd + ' --share-network %s' % share_network
        if availability_zone:
            cmd = cmd + ' --availability-zone %s' % availability_zone
        if share_group:
            cmd = cmd + ' --share-group %s' % share_group

        share_object = self.dict_result('share', cmd)
        self._wait_for_object_status(
            'share', share_object['id'], 'available')

        if add_cleanup:
            self.addCleanup(
                self.openstack, 'share delete %s' % share_object['id']
            )
        return share_object



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest.lib.common.utils import data_utils

from manilaclient.tests.functional import base


@ddt.ddt
class SecurityServiceReadWriteTest(base.BaseTestCase):

    def setUp(self):
        super(SecurityServiceReadWriteTest, self).setUp()
        self.name = data_utils.rand_name('autotest')
        self.description = 'fake_description'
        self.user = 'fake_user'
        self.password = 'fake_password'
        self.server = 'fake_server'
        self.domain = 'fake_domain'
        self.dns_ip = '1.2.3.4'
        self.ou = 'fake_ou'

    @ddt.data(
        {'name': 'test_name'},
        {'description': 'test_description'},
        {'user': 'test_username'},
        {'password': 'test_password'},
        {'server': 'test_server'},
        {'domain': 'test_domain'},
        {'dns_ip': 'test_dns_ip'},
        {'ou': 'test_ou'},
        {'name': '""'},
        {'description': '""'},
        {'user': '""'},
        {'password': '""'},
        {'server': '""'},
        {'domain': '""'},
        {'dns_ip': '""'},
        {'ou': '""'},
    )
    def test_create_update_security_service(self, ss_data):
        expected_data = {
            'name': self.name,
            'description': self.description,
            'user': self.user,
            'password': self.password,
            'server': self.server,
            'domain': self.domain,
            'dns_ip': self.dns_ip,
            'ou': self.ou,
        }

        ss = self.create_security_service(**expected_data)
        update = self.admin_client.update_security_service(ss['id'], **ss_data)
        expected_data.update(ss_data)

        for k, v in expected_data.items():
            if v == '""':
                self.assertEqual('None', update[k])
            else:
                self.assertEqual(v, update[k])



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt

from manilaclient.tests.functional import base


@ddt.ddt
class ManilaClientTestServicesReadOnly(base.BaseTestCase):

    @ddt.data("1.0", "2.0", "2.6", "2.7")
    def test_services_list(self, microversion):
        self.skip_if_microversion_not_supported(microversion)
        self.admin_client.manila('service-list', microversion=microversion)

    def test_list_with_debug_flag(self):
        self.clients['admin'].manila('service-list', flags='--debug')

    def test_shares_list_filter_by_host(self):
        self.clients['admin'].manila('service-list', params='--host host')

    def test_shares_list_filter_by_binary(self):
        self.clients['admin'].manila('service-list', params='--binary binary')

    def test_shares_list_filter_by_zone(self):
        self.clients['admin'].manila('service-list', params='--zone zone')

    def test_shares_list_filter_by_status(self):
        self.clients['admin'].manila('service-list', params='--status status')

    def test_shares_list_filter_by_state(self):
        self.clients['admin'].manila('service-list', params='--state state')



# Copyright 2015 Andrew Kerr
# Copyright 2015 Chuck Fouts
# Copyright 2016 Clinton Knight
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient import api_versions
from manilaclient import base
from manilaclient.common.apiclient import base as common_base
from manilaclient.common import constants

RESOURCES_PATH = '/share-groups'
RESOURCE_PATH = '/share-groups/%s'
RESOURCE_PATH_ACTION = '/share-groups/%s/action'
RESOURCES_NAME = 'share_groups'
RESOURCE_NAME = 'share_group'
SG_GRADUATION_VERSION = "2.55"


class ShareGroup(common_base.Resource):
    """A share group is a logical grouping of shares on a single backend."""

    def __repr__(self):
        return "<Share Group: %s>" % self.id

    def update(self, **kwargs):
        """Update this share group."""
        self.manager.update(self, **kwargs)

    def delete(self, force=False):
        """Delete this share group."""
        self.manager.delete(self, force=force)

    def reset_state(self, state):
        """Update this share group with the provided state."""
        self.manager.reset_state(self, state)


class ShareGroupManager(base.ManagerWithFind):
    """Manage :class:`ShareGroup` resources."""
    resource_class = ShareGroup

    def _create_share_group(
            self, share_group_type=None, share_types=None, share_network=None,
            name=None, description=None, source_share_group_snapshot=None,
            availability_zone=None):
        """Create a Share Group.

        :param share_group_type: either instance of ShareGroupType or text
            with UUID
        :param share_types: list of the share types allowed in the group. May
            not be supplied when 'source_group_snapshot_id' is provided.  These
            may be ShareType objects or UUIDs.
        :param share_network: either the share network object or text of the
            UUID - represents the share network to use when creating a
            share group when driver_handles_share_servers = True.
        :param name: text - name of the new share group
        :param description: text - description of the share group
        :param source_share_group_snapshot: text - either instance of
            ShareGroupSnapshot or text with UUID from which this shar_group is
            to be created. May not be supplied when 'share_types' is provided.
        :param availability_zone: name of the availability zone where the
            group is to be created
        :rtype: :class:`ShareGroup`
        """

        if share_types and source_share_group_snapshot:
            raise ValueError('Cannot specify a share group with both'
                             'share_types and source_share_group_snapshot.')

        body = {}

        if name:
            body['name'] = name
        if description:
            body['description'] = description
        if availability_zone:
            body['availability_zone'] = availability_zone
        if share_group_type:
            body['share_group_type_id'] = common_base.getid(share_group_type)
        if share_network:
            body['share_network_id'] = common_base.getid(share_network)

        if source_share_group_snapshot:
            body['source_share_group_snapshot_id'] = common_base.getid(
                source_share_group_snapshot)
        elif share_types:
            body['share_types'] = [common_base.getid(share_type)
                                   for share_type in share_types]

        return self._create(
            RESOURCES_PATH, {RESOURCE_NAME: body}, RESOURCE_NAME)

    @api_versions.wraps("2.31", "2.54")
    @api_versions.experimental_api
    def create(self, share_group_type=None, share_types=None,
               share_network=None, name=None, description=None,
               source_share_group_snapshot=None, availability_zone=None):
        return self._create_share_group(
            share_group_type=share_group_type, share_types=share_types,
            share_network=share_network, name=name, description=description,
            source_share_group_snapshot=source_share_group_snapshot,
            availability_zone=availability_zone)

    @api_versions.wraps(SG_GRADUATION_VERSION)  # noqa
    def create(self, share_group_type=None, share_types=None,  # noqa
               share_network=None, name=None, description=None,
               source_share_group_snapshot=None, availability_zone=None):
        return self._create_share_group(
            share_group_type=share_group_type, share_types=share_types,
            share_network=share_network, name=name, description=description,
            source_share_group_snapshot=source_share_group_snapshot,
            availability_zone=availability_zone)

    def _get_share_group(self, share_group):
        """Get a share group.

        :param share_group: either ShareGroup object or text with its UUID
        :rtype: :class:`ShareGroup`
        """
        share_group_id = common_base.getid(share_group)
        url = RESOURCE_PATH % share_group_id
        return self._get(url, RESOURCE_NAME)

    @api_versions.wraps("2.31", "2.54")
    @api_versions.experimental_api
    def get(self, share_group):
        return self._get_share_group(share_group)

    @api_versions.wraps(SG_GRADUATION_VERSION)  # noqa
    def get(self, share_group):  # noqa
        return self._get_share_group(share_group)

    def _list_share_groups(self, detailed=True, search_opts=None,
                           sort_key=None, sort_dir=None):
        """Get a list of all share groups.

        :param detailed: Whether to return detailed share group info or not.
        :param search_opts: dict with search options to filter out groups.
            available keys include (('name1', 'name2', ...), 'type'):
            - ('offset', int)
            - ('limit', int)
            - ('all_tenants', int)
            - ('name', text)
            - ('status', text)
            - ('share_server_id', text)
            - ('share_group_type_id', text)
            - ('source_share_group_snapshot_id', text)
            - ('host', text)
            - ('share_network_id', text)
            - ('project_id', text)
        :param sort_key: Key to be sorted (i.e. 'created_at' or 'status').
        :param sort_dir: Sort direction, should be 'desc' or 'asc'.
        :rtype: list of :class:`ShareGroup`
        """

        search_opts = search_opts or {}

        if sort_key is not None:
            if sort_key in constants.SHARE_GROUP_SORT_KEY_VALUES:
                search_opts['sort_key'] = sort_key
                # NOTE(cknight): Replace aliases with appropriate keys
                if sort_key == 'share_group_type':
                    search_opts['sort_key'] = 'share_group_type_id'
                elif sort_key == 'share_network':
                    search_opts['sort_key'] = 'share_network_id'
            else:
                msg = 'sort_key must be one of the following: %s.'
                msg_args = ', '.join(constants.SHARE_GROUP_SORT_KEY_VALUES)
                raise ValueError(msg % msg_args)

        if sort_dir is not None:
            if sort_dir in constants.SORT_DIR_VALUES:
                search_opts['sort_dir'] = sort_dir
            else:
                raise ValueError('sort_dir must be one of the following: %s.'
                                 % ', '.join(constants.SORT_DIR_VALUES))

        query_string = self._build_query_string(search_opts)

        if detailed:
            url = RESOURCES_PATH + '/detail' + query_string
        else:
            url = RESOURCES_PATH + query_string

        return self._list(url, RESOURCES_NAME)

    @api_versions.wraps("2.31", "2.54")
    @api_versions.experimental_api
    def list(self, detailed=True, search_opts=None,
             sort_key=None, sort_dir=None):
        return self._list_share_groups(
            detailed=detailed, search_opts=search_opts,
            sort_key=sort_key, sort_dir=sort_dir)

    @api_versions.wraps(SG_GRADUATION_VERSION)  # noqa
    def list(self, detailed=True, search_opts=None,  # noqa
             sort_key=None, sort_dir=None):
        return self._list_share_groups(
            detailed=detailed, search_opts=search_opts, sort_key=sort_key,
            sort_dir=sort_dir)

    def _update_share_group(self, share_group, **kwargs):
        """Updates a share group.

        :param share_group: either ShareGroup object or text with its UUID
        :rtype: :class:`ShareGroup`
        """
        share_group_id = common_base.getid(share_group)
        url = RESOURCE_PATH % share_group_id
        if not kwargs:
            return self._get(url, RESOURCE_NAME)
        else:
            body = {RESOURCE_NAME: kwargs}
            return self._update(url, body, RESOURCE_NAME)

    @api_versions.wraps("2.31", "2.54")
    def update(self, share_group, **kwargs):
        return self._update_share_group(share_group, **kwargs)

    @api_versions.wraps(SG_GRADUATION_VERSION)  # noqa
    def update(self, share_group, **kwargs):  # noqa
        return self._update_share_group(share_group, **kwargs)

    def _delete_share_group(self, share_group, force=False):
        """Delete a share group.

        :param share_group: either ShareGroup object or text with its UUID
        :param force: True to force the deletion
        """
        share_group_id = common_base.getid(share_group)
        if force:
            url = RESOURCE_PATH_ACTION % share_group_id
            body = {'force_delete': None}
            self.api.client.post(url, body=body)
        else:
            url = RESOURCE_PATH % share_group_id
            self._delete(url)

    @api_versions.wraps("2.31", "2.54")
    @api_versions.experimental_api
    def delete(self, share_group, force=False):
        self._delete_share_group(share_group, force=force)

    @api_versions.wraps(SG_GRADUATION_VERSION)  # noqa
    def delete(self, share_group, force=False):  # noqa
        self._delete_share_group(share_group, force=force)

    def _share_group_reset_state(self, share_group, state):
        """Update the specified share group with the provided state.

        :param share_group: either ShareGroup object or text with its UUID
        :param state: The new state for the share group
        """

        share_group_id = common_base.getid(share_group)
        url = RESOURCE_PATH_ACTION % share_group_id
        body = {'reset_status': {'status': state}}
        self.api.client.post(url, body=body)

    @api_versions.wraps("2.31", "2.54")
    @api_versions.experimental_api
    def reset_state(self, share_group, state):
        self._share_group_reset_state(share_group, state)

    @api_versions.wraps(SG_GRADUATION_VERSION)  # noqa
    def reset_state(self, share_group, state):  # noqa
        self._share_group_reset_state(share_group, state)



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions

"""
Exceptions for functional tests.
"""


class ResourceReleaseFailed(exceptions.TempestException):
    message = "Failed to release resource '%(res_type)s' with id '%(res_id)s'."


class InvalidResource(exceptions.TempestException):
    message = "Provided invalid resource: %(message)s"


class InvalidData(exceptions.TempestException):
    message = "Provided invalid data: %(message)s"


class ShareTypeNotFound(exceptions.NotFound):
    message = "Share type '%(share_type)s' was not found."


class InvalidConfiguration(exceptions.TempestException):
    message = "Invalid configuration: %(reason)s"


class ShareBuildErrorException(exceptions.TempestException):
    message = "Share %(share)s failed to build and is in ERROR status."


class ShareReplicaBuildErrorException(exceptions.TempestException):
    message = ("Share replica %(replica)s failed to build and is in ERROR "
               "status.")


class SnapshotBuildErrorException(exceptions.TempestException):
    message = "Snapshot %(snapshot)s failed to build and is in ERROR status."


class AccessRuleCreateErrorException(exceptions.TempestException):
    message = "Access rule %(access)s failed to create and is in ERROR state."


class AccessRuleDeleteErrorException(exceptions.TempestException):
    message = "Access rule %(access)s failed to delete and is in ERROR state."


class ShareMigrationException(exceptions.TempestException):
    message = ("Share %(share_id)s failed to migrate from "
               "host %(src)s to host %(dest)s.")


class ShareServerMigrationException(exceptions.TempestException):
    message = ("Share server %(server_id)s failed to migrate.")



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from osc_lib import exceptions
from osc_lib import utils as oscutils

from manilaclient.osc import utils
from manilaclient.osc.v2 import services as osc_services
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes


class TestShareService(manila_fakes.TestShare):

    def setUp(self):
        super(TestShareService, self).setUp()

        self.services_mock = self.app.client_manager.share.services
        self.services_mock.reset_mock()


class TestShareServiceSet(TestShareService):

    def setUp(self):
        super(TestShareServiceSet, self).setUp()

        self.share_service = (
            manila_fakes.FakeShareService.create_fake_service()
        )
        self.cmd = osc_services.SetShareService(self.app, None)

    def test_share_service_set_enable(self):
        arglist = [
            self.share_service.host,
            self.share_service.binary,
            '--enable'
        ]
        verifylist = [
            ('host', self.share_service.host),
            ('binary', self.share_service.binary),
            ('enable', True)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.services_mock.enable.assert_called_with(
            self.share_service.host,
            self.share_service.binary)
        self.assertIsNone(result)

    def test_share_service_set_enable_exception(self):
        arglist = [
            self.share_service.host,
            self.share_service.binary,
            '--enable'
        ]
        verifylist = [
            ('host', self.share_service.host),
            ('binary', self.share_service.binary),
            ('enable', True)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.services_mock.enable.side_effect = Exception()
        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    def test_share_service_set_disable(self):
        arglist = [
            self.share_service.host,
            self.share_service.binary,
            '--disable'
        ]
        verifylist = [
            ('host', self.share_service.host),
            ('binary', self.share_service.binary),
            ('disable', True)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.services_mock.disable.assert_called_with(
            self.share_service.host,
            self.share_service.binary)
        self.assertIsNone(result)

    def test_share_service_set_disable_exception(self):
        arglist = [
            self.share_service.host,
            self.share_service.binary,
            '--disable'
        ]
        verifylist = [
            ('host', self.share_service.host),
            ('binary', self.share_service.binary),
            ('disable', True)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.services_mock.disable.side_effect = Exception()
        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestShareServiceList(TestShareService):

    columns = [
        'id',
        'binary',
        'host',
        'zone',
        'status',
        'state',
        'updated_at'
    ]

    column_headers = utils.format_column_headers(columns)

    def setUp(self):
        super(TestShareServiceList, self).setUp()

        self.services_list = (
            manila_fakes.FakeShareService.create_fake_services()
        )
        self.services_mock.list.return_value = self.services_list
        self.values = (oscutils.get_dict_properties(
            i._info, self.columns) for i in self.services_list)

        self.cmd = osc_services.ListShareService(self.app, None)

    def test_share_service_list(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.services_mock.list.assert_called_with(
            search_opts={
                'host': None,
                'binary': None,
                'status': None,
                'state': None,
                'zone': None
            })
        self.assertEqual(self.column_headers, columns)
        self.assertEqual(list(self.values), list(data))

    def test_share_service_list_host_status(self):
        arglist = [
            '--host', self.services_list[0].host,
            '--status', self.services_list[1].status
        ]
        verifylist = [
            ('host', self.services_list[0].host),
            ('status', self.services_list[1].status)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.services_mock.list.assert_called_with(
            search_opts={
                'host': self.services_list[0].host,
                'binary': None,
                'status': self.services_list[1].status,
                'state': None,
                'zone': None
            })
        self.assertEqual(self.column_headers, columns)
        self.assertEqual(list(self.values), list(data))

    def test_share_service_list_binary_state_zone(self):
        arglist = [
            '--binary', self.services_list[0].binary,
            '--state', self.services_list[1].state,
            '--zone', self.services_list[1].zone
        ]
        verifylist = [
            ('binary', self.services_list[0].binary),
            ('state', self.services_list[1].state),
            ('zone', self.services_list[1].zone)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.services_mock.list.assert_called_with(
            search_opts={
                'host': None,
                'binary': self.services_list[0].binary,
                'status': None,
                'state': self.services_list[1].state,
                'zone': self.services_list[1].zone
            })
        self.assertEqual(self.column_headers, columns)
        self.assertEqual(list(self.values), list(data))



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import traceback

from oslo_log import log
from tempest.lib.cli import base
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions as lib_exc

from manilaclient.common import constants
from manilaclient import config
from manilaclient.tests.functional import client
from manilaclient.tests.functional import utils

CONF = config.CONF
LOG = log.getLogger(__name__)


class handle_cleanup_exceptions(object):
    """Handle exceptions raised with cleanup operations.

    Always suppress errors when lib_exc.NotFound or lib_exc.Forbidden
    are raised.
    Suppress all other exceptions only in case config opt
    'suppress_errors_in_cleanup' is True.
    """

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        if isinstance(exc_value, (lib_exc.NotFound, lib_exc.Forbidden)):
            return True
        elif CONF.suppress_errors_in_cleanup:
            LOG.error("Suppressed cleanup error: \n%s", traceback.format_exc())
            return True
        return False  # Don't suppress cleanup errors


class BaseTestCase(base.ClientTestBase):

    # Will be cleaned up after test suite run
    class_resources = []

    # Will be cleaned up after single test run
    method_resources = []

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self.addCleanup(self.clear_resources)

    @classmethod
    def tearDownClass(cls):
        super(BaseTestCase, cls).tearDownClass()
        cls.clear_resources(cls.class_resources)

    @classmethod
    def clear_resources(cls, resources=None):
        """Deletes resources, that were created in test suites.

        This method tries to remove resources from resource list,
        if it is not found, assume it was deleted in test itself.
        It is expected, that all resources were added as LIFO
        due to restriction of deletion resources, that are in the chain.
        :param resources: dict with keys 'type','id','client',
            'deletion_params' and 'deleted'. Optional 'deletion_params'
            contains additional data needed to delete some type of resources.
            Ex:
            params = {
                'type': 'share_network_subnet',
                'id': 'share-network-subnet-id',
                'client': None,
                'deletion_params': {
                    'share_network': 'share-network-id',
                },
                'deleted': False,
            }
        """

        if resources is None:
            resources = cls.method_resources
        for res in resources:
            if "deleted" not in res:
                res["deleted"] = False
            if "client" not in res:
                res["client"] = cls.get_cleanup_client()
            if not(res["deleted"]):
                res_id = res["id"]
                client = res["client"]
                deletion_params = res.get("deletion_params")
                with handle_cleanup_exceptions():
                    # TODO(vponomaryov): add support for other resources
                    if res["type"] == "share_type":
                        client.delete_share_type(
                            res_id, microversion=res["microversion"])
                        client.wait_for_share_type_deletion(
                            res_id, microversion=res["microversion"])
                    elif res["type"] == "share_network":
                        client.delete_share_network(
                            res_id, microversion=res["microversion"])
                        client.wait_for_share_network_deletion(
                            res_id, microversion=res["microversion"])
                    elif res["type"] == "share_network_subnet":
                        client.delete_share_network_subnet(
                            share_network_subnet=res_id,
                            share_network=deletion_params["share_network"],
                            microversion=res["microversion"])
                        client.wait_for_share_network_subnet_deletion(
                            share_network_subnet=res_id,
                            share_network=deletion_params["share_network"],
                            microversion=res["microversion"])
                    elif res["type"] == "share":
                        client.delete_share(
                            res_id, microversion=res["microversion"])
                        client.wait_for_share_deletion(
                            res_id, microversion=res["microversion"])
                    elif res["type"] == "snapshot":
                        client.delete_snapshot(
                            res_id, microversion=res["microversion"])
                        client.wait_for_snapshot_deletion(
                            res_id, microversion=res["microversion"])
                    elif res["type"] == "share_replica":
                        client.delete_share_replica(
                            res_id, microversion=res["microversion"])
                        client.wait_for_share_replica_deletion(
                            res_id, microversion=res["microversion"])
                    else:
                        LOG.warning("Provided unsupported resource type for "
                                    "cleanup '%s'. Skipping.", res["type"])
                res["deleted"] = True

    @classmethod
    def get_admin_client(cls):
        manilaclient = client.ManilaCLIClient(
            username=CONF.admin_username,
            password=CONF.admin_password,
            tenant_name=CONF.admin_tenant_name,
            project_domain_name=CONF.admin_project_domain_name or None,
            project_domain_id=CONF.admin_project_domain_id or None,
            user_domain_name=CONF.admin_user_domain_name or None,
            user_domain_id=CONF.admin_user_domain_id or None,
            uri=CONF.admin_auth_url or CONF.auth_url,
            insecure=CONF.insecure,
            cli_dir=CONF.manila_exec_dir)
        # Set specific for admin project share network
        manilaclient.share_network = CONF.admin_share_network
        return manilaclient

    @classmethod
    def get_user_client(cls):
        manilaclient = client.ManilaCLIClient(
            username=CONF.username,
            password=CONF.password,
            tenant_name=CONF.tenant_name,
            project_domain_name=CONF.project_domain_name or None,
            project_domain_id=CONF.project_domain_id or None,
            user_domain_name=CONF.user_domain_name or None,
            user_domain_id=CONF.user_domain_id or None,
            uri=CONF.auth_url,
            insecure=CONF.insecure,
            cli_dir=CONF.manila_exec_dir)
        # Set specific for user project share network
        manilaclient.share_network = CONF.share_network
        return manilaclient

    @property
    def admin_client(self):
        if not hasattr(self, '_admin_client'):
            self._admin_client = self.get_admin_client()
        return self._admin_client

    @property
    def user_client(self):
        if not hasattr(self, '_user_client'):
            self._user_client = self.get_user_client()
        return self._user_client

    def _get_clients(self):
        return {'admin': self.admin_client, 'user': self.user_client}

    def skip_if_microversion_not_supported(self, microversion):
        if not utils.is_microversion_supported(microversion):
            raise self.skipException(
                "Microversion '%s' is not supported." % microversion)

    @classmethod
    def create_share_type(cls, name=None, driver_handles_share_servers=True,
                          snapshot_support=None,
                          create_share_from_snapshot=None,
                          revert_to_snapshot=None, mount_snapshot=None,
                          is_public=True, client=None, cleanup_in_class=False,
                          microversion=None, extra_specs=None,
                          description=None):
        if client is None:
            client = cls.get_admin_client()
        data = {
            "name": name,
            "driver_handles_share_servers": driver_handles_share_servers,
            "snapshot_support": snapshot_support,
            "is_public": is_public,
            "microversion": microversion,
            "extra_specs": extra_specs,
            "create_share_from_snapshot": create_share_from_snapshot,
            "revert_to_snapshot": revert_to_snapshot,
            "mount_snapshot": mount_snapshot,
        }
        if description:
            data["description"] = description
        share_type = client.create_share_type(**data)
        resource = {
            "type": "share_type",
            "id": share_type["ID"],
            "client": client,
            "microversion": microversion,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return share_type

    @classmethod
    def update_share_type(cls, share_type_id, name=None,
                          is_public=None, client=None,
                          microversion=None, description=None):
        if client is None:
            client = cls.get_admin_client()
        data = {
            "share_type_id": share_type_id,
            "microversion": microversion,
        }
        if name is not None:
            data["name"] = name
        if description is not None:
            data["description"] = description
        if is_public is not None:
            data["is_public"] = is_public
        share_type = client.update_share_type(**data)
        return share_type

    @classmethod
    def create_share_network(cls, name=None, description=None,
                             neutron_net_id=None,
                             neutron_subnet_id=None,
                             availability_zone=None, client=None,
                             cleanup_in_class=False, microversion=None):
        if client is None:
            client = cls.get_admin_client()
        share_network = client.create_share_network(
            name=name,
            description=description,
            neutron_net_id=neutron_net_id,
            neutron_subnet_id=neutron_subnet_id,
            availability_zone=availability_zone,
            microversion=microversion,
        )
        resource = {
            "type": "share_network",
            "id": share_network["id"],
            "client": client,
            "microversion": microversion,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return share_network

    @classmethod
    def add_share_network_subnet(cls, share_network,
                                 neutron_net_id=None, neutron_subnet_id=None,
                                 availability_zone=None, client=None,
                                 cleanup_in_class=False, microversion=None):
        if client is None:
            client = cls.get_admin_client()
        share_network_subnet = client.add_share_network_subnet(
            share_network, neutron_net_id, neutron_subnet_id,
            availability_zone)
        resource = {
            "type": "share_network_subnet",
            "id": share_network_subnet["id"],
            "client": client,
            "deletion_params": {
                "share_network": share_network,
            },
            "microversion": microversion,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return share_network_subnet

    @classmethod
    def create_share(cls, share_protocol=None, size=None, share_network=None,
                     share_type=None, name=None, description=None,
                     public=False, snapshot=None, metadata=None,
                     client=None, use_wait_option=False,
                     cleanup_in_class=False, wait_for_creation=True,
                     microversion=None):
        client = client or cls.get_admin_client()
        data = {
            'share_protocol': share_protocol or client.share_protocol,
            'size': size or 1,
            'name': name,
            'description': description,
            'public': public,
            'snapshot': snapshot,
            'metadata': metadata,
            'microversion': microversion,
            'wait': use_wait_option,
        }

        share_type = share_type or CONF.share_type
        share_network = share_network or cls._determine_share_network_to_use(
            client, share_type, microversion=microversion)

        data['share_type'] = share_type
        data['share_network'] = share_network
        share = client.create_share(**data)
        resource = {
            "type": "share",
            "id": share["id"],
            "client": client,
            "microversion": microversion,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        if wait_for_creation and not use_wait_option:
            client.wait_for_resource_status(share['id'],
                                            constants.STATUS_AVAILABLE)
        return share

    @classmethod
    def delete_share(cls, shares_to_delete, share_group_id=None,
                     wait=False, client=None, microversion=None):
        client = client or cls.get_admin_client()
        client.delete_share(shares_to_delete, share_group_id=share_group_id,
                            wait=wait, microversion=microversion)

    @classmethod
    def _determine_share_network_to_use(cls, client, share_type,
                                        microversion=None):
        """Determine what share network we need from the share type."""

        # Get share type, determine if we need the share network
        share_type = client.get_share_type(share_type,
                                           microversion=microversion)
        dhss_pattern = re.compile('driver_handles_share_servers : ([a-zA-Z]+)')
        dhss = dhss_pattern.search(share_type['required_extra_specs']).group(1)
        return client.share_network if dhss.lower() == 'true' else None

    @classmethod
    def create_security_service(cls, type='ldap', name=None, description=None,
                                dns_ip=None, ou=None, server=None, domain=None,
                                user=None, password=None, client=None,
                                cleanup_in_class=False, microversion=None):
        if client is None:
            client = cls.get_admin_client()
        data = {
            'type': type,
            'name': name,
            'description': description,
            'user': user,
            'password': password,
            'server': server,
            'domain': domain,
            'dns_ip': dns_ip,
            'ou': ou,
            'microversion': microversion,
        }
        ss = client.create_security_service(**data)
        resource = {
            "type": "share",
            "id": ss["id"],
            "client": client,
            "microversion": microversion,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return ss

    @classmethod
    def create_snapshot(cls, share, name=None, description=None,
                        force=False, client=None, wait_for_creation=True,
                        cleanup_in_class=False, microversion=None):
        if client is None:
            client = cls.get_admin_client()
        data = {
            'share': share,
            'name': name,
            'description': description,
            'force': force,
            'microversion': microversion,
        }
        snapshot = client.create_snapshot(**data)
        resource = {
            "type": "snapshot",
            "id": snapshot["id"],
            "client": client,
            "microversion": microversion,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        if wait_for_creation:
            client.wait_for_snapshot_status(snapshot['id'], 'available')
        return snapshot

    @classmethod
    def create_message(cls, client=None, wait_for_creation=True,
                       cleanup_in_class=False, microversion=None):
        """Trigger a 'no valid host' situation to generate a message."""
        if client is None:
            client = cls.get_admin_client()

        extra_specs = {
            'vendor_name': 'foobar',
        }
        share_type_name = data_utils.rand_name("share-type")
        cls.create_share_type(
            name=share_type_name, extra_specs=extra_specs,
            driver_handles_share_servers=False, client=client,
            cleanup_in_class=cleanup_in_class, microversion=microversion)

        share_name = data_utils.rand_name("share")
        share = cls.create_share(
            name=share_name, share_type=share_type_name,
            cleanup_in_class=cleanup_in_class, microversion=microversion,
            wait_for_creation=False, client=client)

        client.wait_for_resource_status(share['id'], constants.STATUS_ERROR)
        message = client.wait_for_message(share['id'])

        resource = {
            "type": "message",
            "id": message["ID"],
            "client": client,
            "microversion": microversion,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return message

    @classmethod
    def create_share_replica(cls, share_id, client=None,
                             wait_for_creation=True, cleanup_in_class=False,
                             microversion=None):
        client = client or cls.get_user_client()

        share_replica = client.create_share_replica(
            share_id, microversion=microversion)
        if wait_for_creation:
            share_replica = client.wait_for_share_replica_status(
                share_replica['id'])

        resource = {
            "type": "share_replica",
            "id": share_replica["id"],
            "client": client,
            "microversion": microversion,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return share_replica



# Copyright (c) 2013 OpenStack Foundation
# Copyright (c) 2015 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

from manilaclient import api_versions
from manilaclient.tests.unit import utils
from manilaclient.v2 import share_type_access

PROJECT_UUID = '11111111-1111-1111-111111111111'


@ddt.ddt
class TypeAccessTest(utils.TestCase):

    def _get_share_type_access_manager(self, microversion):
        version = api_versions.APIVersion(microversion)
        mock_microversion = mock.Mock(api_version=version)
        return share_type_access.ShareTypeAccessManager(
            api=mock_microversion)

    @ddt.data(
        ("1.0", "os-share-type-access"),
        ("2.0", "os-share-type-access"),
        ("2.6", "os-share-type-access"),
        ("2.7", "share_type_access"),
    )
    @ddt.unpack
    def test_list(self, microversion, action_name):
        fake_access_list = ['foo', 'bar']
        share_type = mock.Mock()
        share_type.uuid = '3'
        share_type.is_public = False
        manager = self._get_share_type_access_manager(microversion)

        with mock.patch.object(manager, '_list',
                               mock.Mock(return_value=fake_access_list)):
            access = manager.list(share_type=share_type, search_opts=None)

            manager._list.assert_called_once_with(
                "/types/3/%s" % action_name, "share_type_access")
            self.assertEqual(fake_access_list, access)

    @ddt.data("1.0", "2.0", "2.6", "2.7")
    def test_list_public(self, microversion):
        share_type = mock.Mock()
        share_type.uuid = '4'
        share_type.is_public = True
        manager = self._get_share_type_access_manager(microversion)

        with mock.patch.object(manager, '_list',
                               mock.Mock(return_value='fake')):
            access = manager.list(share_type=share_type)

            self.assertFalse(manager._list.called)
            self.assertIsNone(access)

    @ddt.data("1.0", "2.0", "2.6", "2.7")
    def test_add_project_access(self, microversion):
        share_type = mock.Mock()
        manager = self._get_share_type_access_manager(microversion)

        with mock.patch.object(manager, '_action',
                               mock.Mock(return_value='fake_action')):
            manager.add_project_access(share_type, PROJECT_UUID)

            manager._action.assert_called_once_with(
                'addProjectAccess', share_type, {'project': PROJECT_UUID})

    @ddt.data("1.0", "2.0", "2.6", "2.7")
    def test_remove_project_access(self, microversion):
        share_type = mock.Mock()
        manager = self._get_share_type_access_manager(microversion)

        with mock.patch.object(manager, '_action',
                               mock.Mock(return_value='fake_action')):
            manager.remove_project_access(share_type, PROJECT_UUID)

            manager._action.assert_called_once_with(
                'removeProjectAccess', share_type, {'project': PROJECT_UUID})



# Copyright 2016 Clinton Knight
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

import manilaclient
from manilaclient import exceptions
from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes as fake
from manilaclient.v2 import share_group_snapshots as snapshots


class ShareGroupSnapshotTest(utils.TestCase):

    def setUp(self):
        super(ShareGroupSnapshotTest, self).setUp()
        self.manager = snapshots.ShareGroupSnapshotManager(fake.FakeClient())
        self.share_group_snapshot = snapshots.ShareGroupSnapshot(
            self.manager, {'id': 'fake_id'})
        self.fake_kwargs = {'key': 'value'}

    def test_repr(self):
        result = str(self.share_group_snapshot)

        self.assertEqual('<Share Group Snapshot: fake_id>', result)

    def test_update(self):
        mock_manager_update = self.mock_object(self.manager, 'update')

        self.share_group_snapshot.update(**self.fake_kwargs)

        mock_manager_update.assert_called_once_with(
            self.share_group_snapshot, **self.fake_kwargs)

    def test_delete(self):
        mock_manager_delete = self.mock_object(self.manager, 'delete')

        self.share_group_snapshot.delete()

        mock_manager_delete.assert_called_once_with(self.share_group_snapshot)

    def test_reset_state(self):
        mock_manager_reset_state = self.mock_object(
            self.manager, 'reset_state')

        self.share_group_snapshot.reset_state('fake_state')

        mock_manager_reset_state.assert_called_once_with(
            self.share_group_snapshot, 'fake_state')


@ddt.ddt
class ShareGroupSnapshotManagerTest(utils.TestCase):

    def setUp(self):
        super(ShareGroupSnapshotManagerTest, self).setUp()
        self.manager = snapshots.ShareGroupSnapshotManager(fake.FakeClient())

    def test_create(self):
        fake_share_group_snapshot = fake.ShareGroupSnapshot()
        mock_create = self.mock_object(
            self.manager, '_create',
            mock.Mock(return_value=fake_share_group_snapshot))
        create_args = {
            'name': fake.ShareGroupSnapshot.name,
            'description': fake.ShareGroupSnapshot.description,
        }

        result = self.manager.create(fake.ShareGroupSnapshot, **create_args)

        self.assertIs(fake_share_group_snapshot, result)
        expected_body = {
            snapshots.RESOURCE_NAME: {
                'name': fake.ShareGroupSnapshot.name,
                'description': fake.ShareGroupSnapshot.description,
                'share_group_id': fake.ShareGroupSnapshot().id,
            },
        }
        mock_create.assert_called_once_with(
            snapshots.RESOURCES_PATH, expected_body, snapshots.RESOURCE_NAME)

    def test_create_minimal_args(self):
        fake_share_group_snapshot = fake.ShareGroupSnapshot()
        mock_create = self.mock_object(
            self.manager, '_create',
            mock.Mock(return_value=fake_share_group_snapshot))

        result = self.manager.create(fake.ShareGroupSnapshot)

        self.assertIs(fake_share_group_snapshot, result)
        expected_body = {
            snapshots.RESOURCE_NAME: {
                'share_group_id': fake.ShareGroupSnapshot().id,
            },
        }
        mock_create.assert_called_once_with(
            snapshots.RESOURCES_PATH, expected_body, snapshots.RESOURCE_NAME)

    def test_create_using_unsupported_microversion(self):
        self.manager.api.api_version = manilaclient.API_MIN_VERSION

        self.assertRaises(
            exceptions.UnsupportedVersion,
            self.manager.create, fake.ShareGroupSnapshot)

    def test_get(self):
        fake_share_group_snapshot = fake.ShareGroupSnapshot()
        mock_get = self.mock_object(
            self.manager, '_get',
            mock.Mock(return_value=fake_share_group_snapshot))

        result = self.manager.get(fake.ShareGroupSnapshot.id)

        self.assertIs(fake_share_group_snapshot, result)
        mock_get.assert_called_once_with(
            snapshots.RESOURCE_PATH % fake.ShareGroupSnapshot.id,
            snapshots.RESOURCE_NAME)

    def test_list(self):
        fake_share_group_snapshot = fake.ShareGroupSnapshot()
        mock_list = self.mock_object(
            self.manager, '_list',
            mock.Mock(return_value=[fake_share_group_snapshot]))

        result = self.manager.list()

        self.assertEqual([fake_share_group_snapshot], result)
        mock_list.assert_called_once_with(
            snapshots.RESOURCES_PATH + '/detail',
            snapshots.RESOURCES_NAME)

    def test_list_no_detail(self):
        fake_share_group_snapshot = fake.ShareGroupSnapshot()
        mock_list = self.mock_object(
            self.manager, '_list',
            mock.Mock(return_value=[fake_share_group_snapshot]))

        result = self.manager.list(detailed=False)

        self.assertEqual([fake_share_group_snapshot], result)
        mock_list.assert_called_once_with(
            snapshots.RESOURCES_PATH, snapshots.RESOURCES_NAME)

    def test_list_with_filters(self):
        fake_share_group_snapshot = fake.ShareGroupSnapshot()
        mock_list = self.mock_object(
            self.manager, '_list',
            mock.Mock(return_value=[fake_share_group_snapshot]))
        filters = {'all_tenants': 1, 'status': 'ERROR'}

        result = self.manager.list(detailed=False, search_opts=filters)

        self.assertEqual([fake_share_group_snapshot], result)
        expected_path = (snapshots.RESOURCES_PATH +
                         '?all_tenants=1&status=ERROR')
        mock_list.assert_called_once_with(
            expected_path, snapshots.RESOURCES_NAME)

    def test_list_with_sorting(self):
        fake_share_group_snapshot = fake.ShareGroupSnapshot()
        mock_list = self.mock_object(
            self.manager, '_list',
            mock.Mock(return_value=[fake_share_group_snapshot]))

        result = self.manager.list(
            detailed=False, sort_dir='asc', sort_key='name')

        self.assertEqual([fake_share_group_snapshot], result)
        expected_path = (
            snapshots.RESOURCES_PATH + '?sort_dir=asc&sort_key=name')
        mock_list.assert_called_once_with(
            expected_path, snapshots.RESOURCES_NAME)

    @ddt.data({'sort_key': 'name', 'sort_dir': 'invalid'},
              {'sort_key': 'invalid', 'sort_dir': 'asc'})
    @ddt.unpack
    def test_list_with_invalid_sorting(self, sort_key, sort_dir):
        self.assertRaises(
            ValueError,
            self.manager.list, sort_dir=sort_dir, sort_key=sort_key)

    def test_update(self):
        fake_share_group_snapshot = fake.ShareGroupSnapshot()
        mock_get = self.mock_object(
            self.manager, '_get',
            mock.Mock(return_value=fake_share_group_snapshot))
        mock_update = self.mock_object(
            self.manager, '_update',
            mock.Mock(return_value=fake_share_group_snapshot))
        update_args = {
            'name': fake.ShareGroupSnapshot.name,
            'description': fake.ShareGroupSnapshot.description,
        }

        result = self.manager.update(fake.ShareGroupSnapshot(), **update_args)

        self.assertIs(fake_share_group_snapshot, result)
        self.assertFalse(mock_get.called)
        mock_update.assert_called_once_with(
            snapshots.RESOURCE_PATH % fake.ShareGroupSnapshot.id,
            {snapshots.RESOURCE_NAME: update_args},
            snapshots.RESOURCE_NAME)

    def test_update_no_data(self):
        fake_share_group_snapshot = fake.ShareGroupSnapshot()
        mock_get = self.mock_object(
            self.manager, '_get',
            mock.Mock(return_value=fake_share_group_snapshot))
        mock_update = self.mock_object(
            self.manager, '_update',
            mock.Mock(return_value=fake_share_group_snapshot))
        update_args = {}

        result = self.manager.update(fake.ShareGroupSnapshot(), **update_args)

        self.assertIs(fake_share_group_snapshot, result)
        mock_get.assert_called_once_with(
            snapshots.RESOURCE_PATH % fake.ShareGroupSnapshot.id,
            snapshots.RESOURCE_NAME)
        self.assertFalse(mock_update.called)

    def test_delete(self):
        mock_delete = self.mock_object(self.manager, '_delete')
        mock_post = self.mock_object(self.manager.api.client, 'post')

        self.manager.delete(fake.ShareGroupSnapshot())

        mock_delete.assert_called_once_with(
            snapshots.RESOURCE_PATH % fake.ShareGroupSnapshot.id)
        self.assertFalse(mock_post.called)

    def test_delete_force(self):
        mock_delete = self.mock_object(self.manager, '_delete')
        mock_post = self.mock_object(self.manager.api.client, 'post')

        self.manager.delete(fake.ShareGroupSnapshot.id, force=True)

        self.assertFalse(mock_delete.called)
        mock_post.assert_called_once_with(
            snapshots.RESOURCE_PATH_ACTION % fake.ShareGroupSnapshot.id,
            body={'force_delete': None})

    def test_reset_state(self):
        mock_post = self.mock_object(self.manager.api.client, 'post')

        self.manager.reset_state(fake.ShareGroupSnapshot(), 'fake_state')

        mock_post.assert_called_once_with(
            snapshots.RESOURCE_PATH_ACTION % fake.ShareGroupSnapshot.id,
            body={'reset_status': {'status': 'fake_state'}})



#   Copyright 2021 Red Hat Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from unittest import mock
import uuid

import ddt
from osc_lib import exceptions
from osc_lib import utils as oscutils

from manilaclient import api_versions
from manilaclient.osc.v2 import share_networks as osc_share_networks
from manilaclient.tests.unit.osc import osc_utils
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes

COLUMNS = ['ID', 'Name']

COLUMNS_DETAIL = [
    'ID',
    'Name',
    'Status',
    'Created At',
    'Updated At',
    'Description',
]


class TestShareNetwork(manila_fakes.TestShare):

    def setUp(self):
        super(TestShareNetwork, self).setUp()
        self.share_networks_mock = self.app.client_manager.share.share_networks
        self.share_networks_mock.reset_mock()
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            api_versions.MAX_VERSION)


@ddt.ddt
class TestShareNetworkCreate(TestShareNetwork):

    def setUp(self):
        super(TestShareNetworkCreate, self).setUp()

        self.share_network = (
            manila_fakes.FakeShareNetwork.create_one_share_network())
        self.share_networks_mock.create.return_value = self.share_network

        self.cmd = osc_share_networks.CreateShareNetwork(self.app, None)

        self.data = self.share_network._info.values()
        self.columns = self.share_network._info.keys()

    @ddt.data('table', 'yaml')
    def test_share_network_create_formatter(self, formatter):
        arglist = [
            '-f', formatter,
        ]
        verifylist = [
            ('formatter', formatter),
        ]
        expected_data = self.share_network._info

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.share_networks_mock.create.assert_called_once_with(
            name=None,
            description=None,
            neutron_net_id=None,
            neutron_subnet_id=None,
            availability_zone=None)

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(expected_data.values(), data)

    def test_share_network_create_with_args(self):
        fake_neutron_net_id = str(uuid.uuid4())
        fake_neutron_subnet_id = str(uuid.uuid4())
        fake_az = mock.Mock()
        fake_az.name = 'nova'
        fake_az.id = str(uuid.uuid4())

        arglist = [
            '--name', 'zorilla-net',
            '--description', 'fastest-backdoor-network-ever',
            '--neutron-net-id', fake_neutron_net_id,
            '--neutron-subnet-id', fake_neutron_subnet_id,
            '--availability-zone', 'nova',
        ]
        verifylist = [
            ('name', 'zorilla-net'),
            ('description', 'fastest-backdoor-network-ever'),
            ('neutron_net_id', fake_neutron_net_id),
            ('neutron_subnet_id', fake_neutron_subnet_id),
            ('availability_zone', 'nova'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.find_resource',
                        return_value=fake_az):
            columns, data = self.cmd.take_action(parsed_args)

        self.share_networks_mock.create.assert_called_once_with(
            name='zorilla-net',
            description='fastest-backdoor-network-ever',
            neutron_net_id=fake_neutron_net_id,
            neutron_subnet_id=fake_neutron_subnet_id,
            availability_zone='nova',
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


@ddt.ddt
class TestShareNetworkDelete(TestShareNetwork):

    def setUp(self):
        super(TestShareNetworkDelete, self).setUp()

        self.share_network = (
            manila_fakes.FakeShareNetwork.create_one_share_network())

        self.share_networks_mock.get.return_value = self.share_network

        self.cmd = osc_share_networks.DeleteShareNetwork(self.app, None)

    def test_share_network_delete_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    @ddt.data(True, False)
    def test_share_network_delete_with_wait(self, wait):
        oscutils.wait_for_delete = mock.Mock(return_value=True)
        share_networks = (
            manila_fakes.FakeShareNetwork.create_share_networks(
                count=2))
        arglist = [
            share_networks[0].id,
            share_networks[1].name,
        ]
        if wait:
            arglist.append('--wait')
        verifylist = [
            ('share_network', [share_networks[0].id, share_networks[1].name])
        ]
        if wait:
            verifylist.append(('wait', True))

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.find_resource',
                        side_effect=share_networks):
            result = self.cmd.take_action(parsed_args)

        self.assertEqual(self.share_networks_mock.delete.call_count,
                         len(share_networks))
        if wait:
            oscutils.wait_for_delete.assert_has_calls([
                mock.call(manager=self.share_networks_mock,
                          res_id=share_networks[0].id),
                mock.call(manager=self.share_networks_mock,
                          res_id=share_networks[1].id)
            ])
        self.assertIsNone(result)

    def test_share_network_delete_exception(self):
        arglist = [
            self.share_network.id,
        ]
        verifylist = [
            ('share_network', [self.share_network.id]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.share_networks_mock.delete.side_effect = exceptions.CommandError()
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)

    def test_share_network_delete_wait_fails(self):
        oscutils.wait_for_delete = mock.Mock(return_value=False)
        arglist = [
            self.share_network.id,
            '--wait',
        ]
        verifylist = [
            ('share_network', [self.share_network.id]),
            ('wait', True),
        ]

        with mock.patch('osc_lib.utils.find_resource',
                        return_value=self.share_network):
            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)
        self.share_networks_mock.delete.assert_called_once_with(
            self.share_network)


@ddt.ddt
class TestShareNetworkShow(TestShareNetwork):

    def setUp(self):
        super(TestShareNetworkShow, self).setUp()

        self.share_network = (
            manila_fakes.FakeShareNetwork.create_one_share_network())
        self.share_networks_mock.get.return_value = self.share_network
        self.security_services_mock = (
            self.app.client_manager.share.security_services)

        self.cmd = osc_share_networks.ShowShareNetwork(self.app, None)

        self.data = self.share_network._info.values()
        self.columns = self.share_network._info.keys()

    def test_share_network_show_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    @ddt.data('name', 'id')
    def test_share_network_show_by(self, attr):
        network_to_show = getattr(self.share_network, attr)
        fake_security_service = mock.Mock()
        fake_security_service.id = str(uuid.uuid4())
        fake_security_service.name = 'security-service-%s' % uuid.uuid4().hex
        self.security_services_mock.list = mock.Mock(
            return_value=[fake_security_service])

        arglist = [
            network_to_show,
        ]
        verifylist = [
            ('share_network', network_to_show),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.find_resource',
                        return_value=self.share_network) as find_resource:

            columns, data = self.cmd.take_action(parsed_args)

            find_resource.assert_called_once_with(
                self.share_networks_mock, network_to_show)
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


@ddt.ddt
class TestShareNetworkList(TestShareNetwork):

    def setUp(self):
        super(TestShareNetworkList, self).setUp()

        self.share_networks = (
            manila_fakes.FakeShareNetwork.create_share_networks(
                count=2))
        self.share_networks_list = oscutils.sort_items(self.share_networks,
                                                       'name:asc',
                                                       str)

        self.share_networks_mock.list.return_value = self.share_networks_list

        self.values = (oscutils.get_dict_properties(
            s._info, COLUMNS) for s in self.share_networks_list)
        self.expected_search_opts = {
            'all_tenants': False,
            'project_id': None,
            'name': None,
            'created_since': None,
            'created_before': None,
            'neutron_net_id': None,
            'neutron_subnet_id': None,
            'network_type': None,
            'segmentation_id': None,
            'cidr': None,
            'ip_version': None,
            'security_service': None,
            'name~': None,
            'description~': None,
            'description': None,
        }

        self.cmd = osc_share_networks.ListShareNetwork(self.app, None)

    @ddt.data(True, False)
    def test_list_share_networks_with_search_opts(self, with_search_opts):
        if with_search_opts:
            arglist = [
                '--name', 'foo',
                '--ip-version', '4',
                '--description~', 'foo-share-network',
            ]
            verifylist = [
                ('name', 'foo'),
                ('ip_version', '4'),
                ('description~', 'foo-share-network'),
            ]
            self.expected_search_opts.update({
                'name': 'foo',
                'ip_version': '4',
                'description~': 'foo-share-network',
            })
        else:
            arglist = []
            verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.share_networks_mock.list.assert_called_once_with(
            search_opts=self.expected_search_opts)
        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(self.values), list(data))

    def test_list_share_networks_all_projects(self):
        all_tenants_list = COLUMNS.copy()
        all_tenants_list.append('Project ID')
        self.expected_search_opts.update({'all_tenants': True})
        list_values = (oscutils.get_dict_properties(
            s._info, all_tenants_list) for s in self.share_networks_list)

        arglist = [
            '--all-projects',
        ]

        verifylist = [
            ('all_projects', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.share_networks_mock.list.assert_called_once_with(
            search_opts=self.expected_search_opts)

        self.assertEqual(all_tenants_list, columns)
        self.assertEqual(list(list_values), list(data))

    def test_list_share_networks_detail(self):
        values = (oscutils.get_dict_properties(
            s._info, COLUMNS_DETAIL) for s in self.share_networks_list)

        arglist = [
            '--detail',
        ]

        verifylist = [
            ('detail', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.share_networks_mock.list.assert_called_once_with(
            search_opts=self.expected_search_opts)
        self.assertEqual(COLUMNS_DETAIL, columns)
        self.assertEqual(list(values), list(data))

    def test_list_share_networks_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.35")

        arglist = [
            '--description',
            'Description',
        ]
        verifylist = [
            ('description', 'Description'),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


@ddt.ddt
class TestShareNetworkUnset(TestShareNetwork):

    def setUp(self):
        super(TestShareNetworkUnset, self).setUp()

        self.share_network = (
            manila_fakes.FakeShareNetwork.create_one_share_network())

        self.share_networks_mock.get.return_value = self.share_network
        self.cmd = osc_share_networks.UnsetShareNetwork(self.app, None)

    def test_unset_share_network_name(self):
        arglist = [
            self.share_network.id,
            '--name',
        ]
        verifylist = [
            ('share_network', self.share_network.id),
            ('name', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.find_resource',
                        return_value=self.share_network):
            result = self.cmd.take_action(parsed_args)

        self.share_networks_mock.update.assert_called_once_with(
            self.share_network, name='')
        self.assertIsNone(result)

    def test_unset_share_network_description(self):
        arglist = [
            self.share_network.id,
            '--description',
        ]
        verifylist = [
            ('share_network', self.share_network.id),
            ('description', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.share_networks_mock.update.assert_called_once_with(
            self.share_network, description='')
        self.assertIsNone(result)

    @ddt.data('name', 'security_service')
    def test_unset_share_network_exception_while_updating(self, attr):
        arglist = [
            self.share_network.id,
            '--name',
            '--security-service',
            'fake-security-service',
        ]
        verifylist = [
            ('share_network', self.share_network.id),
            ('name', True),
            ('security_service', 'fake-security-service'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        if attr == 'name':
            self.share_networks_mock.update.side_effect = Exception()
        else:
            self.share_networks_mock.remove_security_service.side_effect = (
                Exception())

        with mock.patch('osc_lib.utils.find_resource',
                        side_effect=[self.share_network,
                                     'fake-security-service']):
            self.assertRaises(exceptions.CommandError,
                              self.cmd.take_action,
                              parsed_args)
        self.share_networks_mock.update.assert_called_once_with(
            self.share_network, name='')
        if attr == 'security_service':
            self.share_networks_mock.remove_security_service\
                .assert_called_once_with(self.share_network,
                                         'fake-security-service')

    def test_unset_share_network_security_service(self):
        arglist = [
            self.share_network.id,
            '--security-service',
            'fake-security-service',
        ]
        verifylist = [
            ('share_network', self.share_network.id),
            ('security_service', 'fake-security-service'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.find_resource',
                        side_effect=[self.share_network,
                                     'fake-security-service']):
            result = self.cmd.take_action(parsed_args)

        self.assertIsNone(result)
        self.share_networks_mock.remove_security_service\
            .assert_called_once_with(self.share_network,
                                     'fake-security-service')


@ddt.ddt
class TestShareNetworkSet(TestShareNetwork):

    def setUp(self):
        super(TestShareNetworkSet, self).setUp()

        self.share_network = (
            manila_fakes.FakeShareNetwork.create_one_share_network())

        self.share_networks_mock.get.return_value = self.share_network

        self.cmd = osc_share_networks.SetShareNetwork(self.app, None)

    @ddt.data({'status': 'error',
               'current_security_service': str(uuid.uuid4()),
               'check_only': True,
               'restart_check': True},
              {'status': None,
               'current_security_service': str(uuid.uuid4()),
               'check_only': True,
               'restart_check': None},
              {'status': None,
               'current_security_service': str(uuid.uuid4()),
               'check_only': True,
               'restart_check': True},
              )
    @ddt.unpack
    def test_set_share_network_api_version_exception(self,
                                                     status,
                                                     current_security_service,
                                                     check_only,
                                                     restart_check):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.62")

        arglist = [self.share_network.id]
        verifylist = [('share_network', self.share_network.id)]
        if status:
            arglist.extend(['--status', status])
            verifylist.append(('status', status))
        if current_security_service:
            arglist.extend(['--current-security-service',
                            current_security_service])
            verifylist.append(('current_security_service',
                               current_security_service))
        if check_only and restart_check:
            arglist.extend(['--check-only', '--restart-check'])
            verifylist.extend([('check_only', True), ('restart_check', True)])

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    def test_set_network_properties(self):
        new_name = 'share-network-name-' + uuid.uuid4().hex
        new_description = 'share-network-description-' + uuid.uuid4().hex
        new_neutron_subnet_id = str(uuid.uuid4())

        arglist = [
            self.share_network.id,
            '--name', new_name,
            '--description', new_description,
            '--neutron-subnet-id', new_neutron_subnet_id,
        ]
        verifylist = [
            ('share_network', self.share_network.id),
            ('name', new_name),
            ('description', new_description),
            ('neutron_subnet_id', new_neutron_subnet_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.find_resource',
                        return_value=self.share_network):
            result = self.cmd.take_action(parsed_args)

        self.share_networks_mock.update.assert_called_once_with(
            self.share_network,
            name=parsed_args.name,
            description=new_description,
            neutron_subnet_id=new_neutron_subnet_id,
        )
        self.assertIsNone(result)

    def test_set_share_network_status(self):
        arglist = [
            self.share_network.id,
            '--status', 'error'
        ]
        verifylist = [
            ('share_network', self.share_network.id),
            ('status', 'error')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.find_resource',
                        return_value=self.share_network):
            result = self.cmd.take_action(parsed_args)

        self.share_networks_mock.reset_state.assert_called_once_with(
            self.share_network, parsed_args.status)
        self.assertIsNone(result)

    def test_set_network_update_exception(self):
        share_network_name = 'share-network-name-' + uuid.uuid4().hex
        arglist = [
            self.share_network.id,
            '--name', share_network_name
        ]
        verifylist = [
            ('share_network', self.share_network.id),
            ('name', share_network_name)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.share_networks_mock.update.side_effect = Exception()

        with mock.patch('osc_lib.utils.find_resource',
                        return_value=self.share_network):
            self.assertRaises(exceptions.CommandError,
                              self.cmd.take_action,
                              parsed_args)
        self.share_networks_mock.update.assert_called_once_with(
            self.share_network, name=parsed_args.name)

    def test_set_share_network_status_exception(self):
        arglist = [
            self.share_network.id,
            '--status', 'error'
        ]
        verifylist = [
            ('share_network', self.share_network.id),
            ('status', 'error')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.share_networks_mock.reset_state.side_effect = Exception()

        with mock.patch('osc_lib.utils.find_resource',
                        return_value=self.share_network):
            self.assertRaises(exceptions.CommandError,
                              self.cmd.take_action,
                              parsed_args)
        self.share_networks_mock.reset_state.assert_called_once_with(
            self.share_network, parsed_args.status)

    @ddt.data({'check_only': False, 'restart_check': False},
              {'check_only': True, 'restart_check': True},
              {'check_only': True, 'restart_check': False})
    @ddt.unpack
    def test_set_share_network_add_new_security_service_check_reset(
            self, check_only, restart_check):
        self.share_networks_mock .add_security_service_check = mock.Mock(
            return_value=(200, {'compatible': True}))

        arglist = [
            self.share_network.id,
            '--new-security-service', 'new-security-service-name',
        ]
        verifylist = [
            ('share_network', self.share_network.id),
            ('new_security_service', 'new-security-service-name'),
        ]

        if check_only:
            arglist.append('--check-only')
            verifylist.append(('check_only', True))
        if restart_check:
            arglist.append('--restart-check')
            verifylist.append(('restart_check', True))

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.find_resource',
                        side_effect=[self.share_network,
                                     'new-security-service']):
            result = self.cmd.take_action(parsed_args)

        if check_only:
            self.share_networks_mock.add_security_service_check\
                .assert_called_once_with(self.share_network,
                                         'new-security-service',
                                         reset_operation=restart_check)
            self.share_networks_mock.add_security_service.assert_not_called()
        else:
            self.share_networks_mock.add_security_service_check\
                .assert_not_called()
            self.share_networks_mock.add_security_service\
                .assert_called_once_with(self.share_network,
                                         'new-security-service')
        self.assertIsNone(result)

    @ddt.data({'check_only': False, 'restart_check': False},
              {'check_only': True, 'restart_check': True},
              {'check_only': True, 'restart_check': False})
    @ddt.unpack
    def test_set_share_network_update_security_service_check_reset(
            self, check_only, restart_check):
        self.share_networks_mock\
            .update_share_network_security_service_check = mock.Mock(
                return_value=(200, {'compatible': True}))

        arglist = [
            self.share_network.id,
            '--new-security-service', 'new-security-service-name',
            '--current-security-service', 'current-security-service-name'
        ]
        verifylist = [
            ('share_network', self.share_network.id),
            ('new_security_service', 'new-security-service-name'),
            ('current_security_service', 'current-security-service-name'),
        ]
        if check_only:
            arglist.append('--check-only')
            verifylist.append(('check_only', True))
        if restart_check:
            arglist.append('--restart-check')
            verifylist.append(('restart_check', True))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.find_resource',
                        side_effect=[self.share_network,
                                     'new-security-service',
                                     'current-security-service']):
            result = self.cmd.take_action(parsed_args)

        if check_only:
            self.share_networks_mock\
                .update_share_network_security_service_check\
                .assert_called_once_with(self.share_network,
                                         'current-security-service',
                                         'new-security-service',
                                         reset_operation=restart_check)
            self.share_networks_mock.update_share_network_security_service\
                .assert_not_called()
        else:
            self.share_networks_mock\
                .update_share_network_security_service_check\
                .assert_not_called()
            self.share_networks_mock.update_share_network_security_service\
                .assert_called_once_with(self.share_network,
                                         'current-security-service',
                                         'new-security-service')
        self.assertIsNone(result)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import logging

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils

from manilaclient import api_versions
from manilaclient.common._i18n import _
from manilaclient.common.apiclient import utils as apiutils
from manilaclient.common import cliutils


LOG = logging.getLogger(__name__)


class ShareInstanceDelete(command.Command):
    """Forces the deletion of the share instance."""
    _description = _("Forces the deletion of a share instance")

    log = logging.getLogger(__name__ + ".ShareInstanceDelete")

    def get_parser(self, prog_name):
        parser = super(ShareInstanceDelete, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar="<instance>",
            nargs="+",
            help=_('ID of the share instance to delete.')
        )
        parser.add_argument(
            "--wait",
            action='store_true',
            default=False,
            help=_("Wait for share instance deletion.")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share
        number_of_deletion_failures = 0

        for instance in parsed_args.instance:
            try:
                share_instance = apiutils.find_resource(
                    share_client.share_instances, instance)

                share_client.share_instances.force_delete(share_instance)

                if parsed_args.wait:
                    if not osc_utils.wait_for_delete(
                            manager=share_client.share_instances,
                            res_id=share_instance.id):
                        number_of_deletion_failures += 1

            except Exception as e:
                number_of_deletion_failures += 1
                LOG.error(_(
                    "Failed to delete a share instance with "
                    "ID '%(instance)s': %(e)s"),
                    {'instance': instance, 'e': e})
        if number_of_deletion_failures > 0:
            msg = (_("%(number_of_deletion_failures)s of "
                     "%(total_of_instances)s instances failed "
                     "to delete.") % {
                'number_of_deletion_failures': number_of_deletion_failures,
                'total_of_instances': len(parsed_args.instance)})
            raise exceptions.CommandError(msg)


class ShareInstanceList(command.Lister):
    """List share instances."""
    _description = _("List share instances")

    def get_parser(self, prog_name):
        parser = super(ShareInstanceList, self).get_parser(prog_name)
        parser.add_argument(
            "--share",
            metavar="<share>",
            default=None,
            help=_("Name or ID of the share to list instances for.")
        )
        parser.add_argument(
            "--export-location",
            metavar="<export-location>",
            default=None,
            help=_("Export location to list instances for.")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        instances = []
        kwargs = {}

        if parsed_args.share:
            # Check if the share exists
            share = osc_utils.find_resource(
                share_client.shares, parsed_args.share)

            instances = share_client.shares.list_instances(share)
        else:
            if share_client.api_version < api_versions.APIVersion("2.35"):
                if parsed_args.export_location:
                    raise exceptions.CommandError(
                        "Filtering by export location is only "
                        "available with manila API version >= 2.35")
            else:
                if parsed_args.export_location:
                    kwargs['export_location'] = parsed_args.export_location
            instances = share_client.share_instances.list(**kwargs)

        columns = [
            'ID',
            'Share ID',
            'Host',
            'Status',
            'Availability Zone',
            'Share Network ID',
            'Share Server ID',
            'Share Type ID',
        ]

        data = (osc_utils.get_dict_properties(
            instance._info, columns) for instance in instances)

        return (columns, data)


class ShareInstanceSet(command.Command):
    """Set share instance"""

    _description = _("Explicitly reset share instance status")

    def get_parser(self, prog_name):
        parser = super(ShareInstanceSet, self).get_parser(prog_name)
        parser.add_argument(
            "instance",
            metavar="<instance>",
            help=_("Instance to be modified.")
        )
        parser.add_argument(
            "--status",
            metavar="<status>",
            help=_('Indicate which state to assign the instance. Options are: '
                   'available, error, creating, deleting,'
                   'error_deleting, migrating, migrating_to, server_migrating.'
                   )
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        instance = osc_utils.find_resource(
            share_client.share_instances,
            parsed_args.instance)

        if parsed_args.status:
            try:
                share_client.share_instances.reset_state(
                    instance,
                    parsed_args.status
                )
            except Exception as e:
                LOG.error(_(
                    "Failed to set status '%(status)s': %(exception)s"),
                    {'status': parsed_args.status, 'exception': e})
                raise exceptions.CommandError(_("Set operation failed"))

        if not instance or not parsed_args.status:
            raise exceptions.CommandError(_(
                "Nothing to set. Please define a '--status'."))


class ShareInstanceShow(command.ShowOne):
    """Show share instance."""
    _description = _("Show share instance")

    def get_parser(self, prog_name):
        parser = super(ShareInstanceShow, self).get_parser(prog_name)
        parser.add_argument(
            "instance",
            metavar="<instance>",
            help=_("ID of the share instance.")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        instance = osc_utils.find_resource(
            share_client.share_instances,
            parsed_args.instance)

        export_locations = share_client.share_instance_export_locations.list(
            instance)

        instance._info['export_locations'] = []
        for export_location in export_locations:
            export_location._info.pop('links', None)
            instance._info['export_locations'].append(export_location._info)

        if parsed_args.formatter == 'table':
            instance._info['export_locations'] = (
                cliutils.convert_dict_list_to_string(
                    instance._info['export_locations']))

        instance._info.pop('links', None)

        return self.dict2columns(instance._info)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

from manilaclient import api_versions
from manilaclient.tests.unit import utils
from manilaclient.v2 import services


@ddt.ddt
class ServicesTest(utils.TestCase):

    def _get_manager(self, microversion):
        version = api_versions.APIVersion(microversion)
        mock_microversion = mock.Mock(api_version=version)
        return services.ServiceManager(api=mock_microversion)

    def _get_resource_path(self, microversion):
        if (api_versions.APIVersion(microversion) >
                api_versions.APIVersion("2.6")):
            return services.RESOURCE_PATH
        return services.RESOURCE_PATH_LEGACY

    @ddt.data("2.6", "2.7")
    def test_list(self, microversion):
        manager = self._get_manager(microversion)
        resource_path = self._get_resource_path(microversion)
        with mock.patch.object(manager, '_list',
                               mock.Mock(return_value='fake')):
            result = manager.list()

            manager._list.assert_called_once_with(
                resource_path, services.RESOURCE_NAME)
            self.assertEqual("fake", result)

    def test_list_services_with_one_search_opt(self):
        manager = self._get_manager("2.7")
        host = 'fake_host'
        query_string = "?host=%s" % host
        with mock.patch.object(manager, '_list',
                               mock.Mock(return_value=None)):
            manager.list({'host': host})
            manager._list.assert_called_once_with(
                services.RESOURCE_PATH + query_string,
                services.RESOURCE_NAME,
            )

    def test_list_services_with_two_search_opts(self):
        manager = self._get_manager("2.7")
        host = 'fake_host'
        binary = 'fake_binary'
        query_string = "?binary=%s&host=%s" % (binary, host)
        with mock.patch.object(manager, '_list',
                               mock.Mock(return_value=None)):
            manager.list({'binary': binary, 'host': host})
            manager._list.assert_called_once_with(
                services.RESOURCE_PATH + query_string,
                services.RESOURCE_NAME,
            )

    @ddt.data("2.6", "2.7")
    def test_enable_service(self, microversion):
        manager = self._get_manager(microversion)
        host = 'fake_host'
        binary = 'fake_binary'
        with mock.patch.object(manager, '_update'):
            manager.enable(binary=binary, host=host)

            manager._update.assert_called_once_with(
                self._get_resource_path(microversion) + '/enable',
                {"host": host, "binary": binary},
            )

    @ddt.data("2.6", "2.7")
    def test_disable_service(self, microversion):
        manager = self._get_manager(microversion)
        host = 'fake_host'
        binary = 'fake_binary'
        with mock.patch.object(manager, '_update'):
            manager.disable(binary=binary, host=host)

            manager._update.assert_called_once_with(
                self._get_resource_path(microversion) + '/disable',
                {"host": host, "binary": binary},
            )



# Copyright 2019 NetApp
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient import base
from manilaclient.common.apiclient import base as common_base

RESOURCES_PATH = '/share-networks/%(share_network_id)s/subnets'
RESOURCE_PATH = RESOURCES_PATH + '/%(share_network_subnet_id)s'
RESOURCE_NAME = 'share_network_subnet'


class ShareNetworkSubnet(common_base.Resource):
    """Network subnet info for Manila share networks."""
    def __repr__(self):
        return "<ShareNetworkSubnet: %s>" % self.id

    def __getitem__(self, key):
        return self._info[key]

    def delete(self):
        """Delete this share network subnet."""
        self.manager.delete(self)


class ShareNetworkSubnetManager(base.ManagerWithFind):
    """Manage :class:`ShareNetworkSubnet` resources."""

    resource_class = ShareNetworkSubnet

    def create(self, neutron_net_id=None, neutron_subnet_id=None,
               availability_zone=None, share_network_id=None):
        """Create share network subnet.

        :param neutron_net_id: ID of Neutron network
        :param neutron_subnet_id: ID of Neutron subnet
        :param availability_zone: Name of the target availability zone
        :rtype: :class:`ShareNetworkSubnet`
        """
        values = {}
        if neutron_net_id:
            values['neutron_net_id'] = neutron_net_id
        if neutron_subnet_id:
            values['neutron_subnet_id'] = neutron_subnet_id
        if availability_zone:
            values['availability_zone'] = availability_zone

        body = {'share-network-subnet': values}
        url = '/share-networks/%(share_network_id)s/subnets' % {
            'share_network_id': share_network_id
        }

        return self._create(url, body, RESOURCE_NAME)

    def get(self, share_network, share_network_subnet):
        """Get a share network subnet.

        :param policy: share network subnet to get.
        :rtype: :class:`NetworkSubnetInfo`
        """
        share_network_id = common_base.getid(share_network)
        share_network_subnet_id = common_base.getid(share_network_subnet)
        url = ('/share-networks/%(share_network_id)s/subnets'
               '/%(share_network_subnet)s') % {
            'share_network_id': share_network_id,
            'share_network_subnet': share_network_subnet_id
        }
        return self._get(url, "share_network_subnet")

    def delete(self, share_network, share_network_subnet):
        """Delete a share network subnet.

        :param share_network: share network that owns the subnet.
        :param share_network_subnet: share network subnet to be deleted.
        """
        url = ('/share-networks/%(share_network_id)s/subnets'
               '/%(share_network_subnet)s') % {
            'share_network_id': common_base.getid(share_network),
            'share_network_subnet': share_network_subnet
        }
        self._delete(url)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

from manilaclient import api_versions
from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes
from manilaclient.v2 import share_replica_export_locations

cs = fakes.FakeClient()


@ddt.ddt
class ShareReplicaExportLocationsTest(utils.TestCase):

    def _get_manager(self, microversion):
        version = api_versions.APIVersion(microversion)
        mock_microversion = mock.Mock(api_version=version)
        return (
            share_replica_export_locations.ShareReplicaExportLocationManager(
                api=mock_microversion)
        )

    def test_list_share_replica_export_locations(self):
        share_replica_id = '1234'
        cs.share_replica_export_locations.list(share_replica_id)
        cs.assert_called(
            'GET', '/share-replicas/%s/export-locations' % share_replica_id)

    def test_get_share_replica_export_location(self):
        share_replica_id = '1234'
        el_uuid = 'fake_el_uuid'
        cs.share_replica_export_locations.get(share_replica_id, el_uuid)
        url = ('/share-replicas/%(share_replica_id)s/export-locations/'
               '%(el_uuid)s')
        payload = {'share_replica_id': share_replica_id, 'el_uuid': el_uuid}
        cs.assert_called('GET', url % payload)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


# python-manilaclient documentation build configuration file

import os
import sys

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
ROOT = os.path.abspath(os.path.join(BASE_DIR, "..", ".."))
sys.path.insert(0, ROOT)

# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc',
              'openstackdocstheme',
              'sphinxcontrib.programoutput',
              'cliff.sphinxext']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
copyright = 'Rackspace, based on work by Jacob Kaplan-Moss'

# List of directories, relative to source directory, that shouldn't be searched
# for source files.
exclude_trees = []

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# -- Options for HTML output --------------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'python-manilaclientdoc'
html_theme = 'openstackdocs'

# openstackdocstheme options
openstackdocs_repo_name = 'openstack/python-manilaclient'
openstackdocs_pdf_link = True
openstackdocs_bug_project = 'python-manilaclient'
openstackdocs_bug_tag = 'docs'

# -- Options for LaTeX output -------------------------------------------------

# Docs job does not install xindy
latex_use_xindy = False

# The paper size ('letter' or 'a4').
# latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
# latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    ('index', 'Manila-client.tex', 'Manila Python Client Documentation',
     'Manila contributors', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# Additional stuff for the LaTeX preamble.
# latex_preamble = ''

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_use_modindex = True



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient import api_versions
from manilaclient import base
from manilaclient.common.apiclient import base as common_base

RESOURCE_PATH_LEGACY = '/os-quota-class-sets'
RESOURCE_PATH = '/quota-class-sets'
REPLICA_QUOTAS_MICROVERSION = "2.53"


class QuotaClassSet(common_base.Resource):

    @property
    def id(self):
        """Needed by base.Resource to self-refresh and be indexed."""
        return self.class_name

    def update(self, *args, **kwargs):
        self.manager.update(self.class_name, *args, **kwargs)


class QuotaClassSetManager(base.ManagerWithFind):
    resource_class = QuotaClassSet

    @api_versions.wraps("1.0", "2.6")
    def get(self, class_name):
        return self._get(
            "%(resource_path)s/%(class_name)s" % {
                "resource_path": RESOURCE_PATH_LEGACY,
                "class_name": class_name},
            "quota_class_set")

    @api_versions.wraps("2.7")  # noqa
    def get(self, class_name):  # noqa
        return self._get(
            "%(resource_path)s/%(class_name)s" % {
                "resource_path": RESOURCE_PATH, "class_name": class_name},
            "quota_class_set")

    def _do_update(self, class_name, shares=None, gigabytes=None,
                   snapshots=None, snapshot_gigabytes=None,
                   share_networks=None, share_replicas=None,
                   replica_gigabytes=None, per_share_gigabytes=None,
                   resource_path=RESOURCE_PATH):
        body = {
            'quota_class_set': {
                'class_name': class_name,
                'shares': shares,
                'snapshots': snapshots,
                'gigabytes': gigabytes,
                'snapshot_gigabytes': snapshot_gigabytes,
                'share_networks': share_networks,
                "share_replicas": share_replicas,
                "replica_gigabytes": replica_gigabytes,
                'per_share_gigabytes': per_share_gigabytes,
            }
        }

        for key in list(body['quota_class_set']):
            if body['quota_class_set'][key] is None:
                body['quota_class_set'].pop(key)

        self._update(
            "%(resource_path)s/%(class_name)s" % {
                "resource_path": resource_path,
                "class_name": class_name},
            body)

    @api_versions.wraps("1.0", "2.6")
    def update(self, class_name, shares=None, gigabytes=None,
               snapshots=None, snapshot_gigabytes=None, share_networks=None):
        return self._do_update(
            class_name, shares=shares, gigabytes=gigabytes,
            snapshots=snapshots, snapshot_gigabytes=snapshot_gigabytes,
            share_networks=share_networks, resource_path=RESOURCE_PATH_LEGACY)

    @api_versions.wraps("2.7", "2.52")  # noqa
    def update(self, class_name, shares=None, gigabytes=None,  # noqa
               snapshots=None, snapshot_gigabytes=None, share_networks=None):
        return self._do_update(
            class_name, shares=shares, gigabytes=gigabytes,
            snapshots=snapshots, snapshot_gigabytes=snapshot_gigabytes,
            share_networks=share_networks, resource_path=RESOURCE_PATH)

    @api_versions.wraps(REPLICA_QUOTAS_MICROVERSION, "2.61")  # noqa
    def update(self, class_name, shares=None, gigabytes=None,  # noqa
               snapshots=None, snapshot_gigabytes=None, share_networks=None,
               share_replicas=None, replica_gigabytes=None):
        return self._do_update(
            class_name, shares=shares, gigabytes=gigabytes,
            snapshots=snapshots, snapshot_gigabytes=snapshot_gigabytes,
            share_networks=share_networks, share_replicas=share_replicas,
            replica_gigabytes=replica_gigabytes, resource_path=RESOURCE_PATH)

    @api_versions.wraps("2.62")  # noqa
    def update(self, class_name, shares=None, gigabytes=None,  # noqa
               snapshots=None, snapshot_gigabytes=None, share_networks=None,
               share_replicas=None, replica_gigabytes=None,
               per_share_gigabytes=None):
        return self._do_update(
            class_name, shares=shares, gigabytes=gigabytes,
            snapshots=snapshots, snapshot_gigabytes=snapshot_gigabytes,
            share_networks=share_networks, share_replicas=share_replicas,
            replica_gigabytes=replica_gigabytes,
            per_share_gigabytes=per_share_gigabytes,
            resource_path=RESOURCE_PATH)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from osc_lib import exceptions
from osc_lib import utils as oscutils

from manilaclient import api_versions
from manilaclient.osc.v2 import share_pools
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes


class TestPool(manila_fakes.TestShare):

    def setUp(self):
        super(TestPool, self).setUp()

        self.pools_mock = self.app.client_manager.share.pools
        self.pools_mock.reset_mock()

        self.share_types_mock = self.app.client_manager.share.share_types
        self.share_types_mock.reset_mock()

        self.app.client_manager.share.api_version = api_versions.APIVersion(
            api_versions.MAX_VERSION)


class TestPoolList(TestPool):

    columns = ['Name', 'Host', 'Backend', 'Pool']

    def setUp(self):
        super(TestPoolList, self).setUp()

        self.share_type = manila_fakes.FakeShareType.create_one_sharetype()
        self.share_types_mock.get.return_value = self.share_type

        self.share_pools = manila_fakes.FakeSharePools.create_share_pools()
        self.pools_mock.list.return_value = self.share_pools

        self.values = (oscutils.get_dict_properties(
            pool._info, self.columns) for pool in self.share_pools)

        self.cmd = share_pools.ListSharePools(self.app, None)

    def test_list_share_pools(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.pools_mock.list.assert_called_with(
            detailed=False,
            search_opts={
                'host': None,
                'backend': None,
                'pool': None,
                'share_type': None,
            })

        self.assertEqual(self.columns, columns)
        self.assertEqual(list(self.values), list(data))

    def test_list_share_pools_filters(self):
        arglist = [
            '--host', self.share_pools[0].host,
            '--backend', self.share_pools[0].backend,
            '--pool', self.share_pools[0].pool
        ]
        verifylist = [
            ('host', self.share_pools[0].host),
            ('backend', self.share_pools[0].backend),
            ('pool', self.share_pools[0].pool)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.pools_mock.list.assert_called_with(
            detailed=False,
            search_opts={
                'host': self.share_pools[0].host,
                'backend': self.share_pools[0].backend,
                'pool': self.share_pools[0].pool,
                'share_type': None,
            })

        self.assertEqual(self.columns, columns)
        self.assertEqual(list(self.values), list(data))

    def test_list_share_pools_share_type(self):
        arglist = [
            '--share-type', self.share_type.id
        ]
        verifylist = [
            ('share_type', self.share_type.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.pools_mock.list.assert_called_with(
            detailed=False,
            search_opts={
                'host': None,
                'backend': None,
                'pool': None,
                'share_type': self.share_type.id,
            })
        self.assertEqual(self.columns, columns)
        self.assertEqual(list(self.values), list(data))

    def test_list_share_pools_share_type_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.22")

        arglist = [
            '--share-type', self.share_type.id
        ]
        verifylist = [
            ('share_type', self.share_type.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_list_share_pools_detail(self):
        detail_columns = ['Name', 'Host', 'Backend', 'Pool', 'Capabilities']
        detail_values = (oscutils.get_dict_properties(
            pool._info, detail_columns) for pool in self.share_pools)
        arglist = [
            '--detail'
        ]
        verifylist = [
            ('detail', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.pools_mock.list.assert_called_with(
            detailed=True,
            search_opts={
                'host': None,
                'backend': None,
                'pool': None,
                'share_type': None,
            })

        self.assertEqual(detail_columns, columns)
        self.assertEqual(list(detail_values), list(data))
